@@ -4,9 +4,7 @@
 extern "C" {
 #endif
 
-#define MAX_ARGS 40
-
-typedef void (*PyUFuncGenericFunction) (char **, intp *, intp *, void *);
+typedef void (*PyUFuncGenericFunction) (char **, npy_intp *, npy_intp *, void *);
 
 typedef struct {
 	PyObject_HEAD
@@ -25,29 +23,24 @@ typedef struct {
 
 #include "arrayobject.h"
 
-#ifdef PY_ARRAY_TYPES_PREFIX
-#  define CAT2(x,y)   x ## y
-#  define CAT(x,y)    CAT2(x,y)
-#  define NS(name)    CAT(PY_ARRAY_TYPES_PREFIX, name)
-#  define intp        NS(intp)
-#endif
-
 #define UFUNC_ERR_IGNORE 0
 #define UFUNC_ERR_WARN   1
 #define UFUNC_ERR_RAISE  2
 #define UFUNC_ERR_CALL   3
+#define UFUNC_ERR_PRINT  4
+#define UFUNC_ERR_LOG    5
 
 	/* Python side integer mask */
 
-#define UFUNC_MASK_DIVIDEBYZERO 0x03
-#define UFUNC_MASK_OVERFLOW 0x0c
-#define UFUNC_MASK_UNDERFLOW 0x30
-#define UFUNC_MASK_INVALID 0xc0
+#define UFUNC_MASK_DIVIDEBYZERO 0x07
+#define UFUNC_MASK_OVERFLOW 0x3f
+#define UFUNC_MASK_UNDERFLOW 0x1ff
+#define UFUNC_MASK_INVALID 0xfff
 
 #define UFUNC_SHIFT_DIVIDEBYZERO 0
-#define UFUNC_SHIFT_OVERFLOW     2
-#define UFUNC_SHIFT_UNDERFLOW    4
-#define UFUNC_SHIFT_INVALID      6       
+#define UFUNC_SHIFT_OVERFLOW     3
+#define UFUNC_SHIFT_UNDERFLOW    6
+#define UFUNC_SHIFT_INVALID      9
 
 
 /* platform-dependent code translates floating point
@@ -57,30 +50,37 @@ typedef struct {
 #define UFUNC_FPE_OVERFLOW      2
 #define UFUNC_FPE_UNDERFLOW     4
 #define UFUNC_FPE_INVALID       8
-	
-#define UFUNC_ERR_DEFAULT 0  /* Default error mode */
+
+#define UFUNC_ERR_DEFAULT  0      /* Error mode that avoids look-up (no checking) */
+
+   /* Default user error mode */
+#define UFUNC_ERR_DEFAULT2                               \
+        (UFUNC_ERR_PRINT << UFUNC_SHIFT_DIVIDEBYZERO) +  \
+        (UFUNC_ERR_PRINT << UFUNC_SHIFT_OVERFLOW) +      \
+        (UFUNC_ERR_PRINT << UFUNC_SHIFT_INVALID)
 
 	/* Only internal -- not exported, yet*/
 typedef struct {
-	/* Multi-iterator portion --- needs to be present in this order 
+	/* Multi-iterator portion --- needs to be present in this order
 	   to work with PyArray_Broadcast */
 	PyObject_HEAD
 	int  numiter;
-	intp size;
-	intp index;
+	npy_intp size;
+	npy_intp index;
 	int nd;
-	intp dimensions[MAX_DIMS];	
-	PyArrayIterObject *iters[MAX_ARGS];
+	npy_intp dimensions[NPY_MAXDIMS];
+	PyArrayIterObject *iters[NPY_MAXARGS];
         /*  End of Multi-iterator portion */
 
 	/* The ufunc */
 	PyUFuncObject *ufunc;
-	
+
 	/* The error handling */
 	int errormask;         /* Integer showing desired error handling */
-	PyObject *errobj;      /* currently a tuple with 
-				  (string, func or None)
+	PyObject *errobj;      /* currently a tuple with
+				  (string, func or obj with write method or None)
 			       */
+        int first;
 
 	/* Specific function and data to use */
 	PyUFuncGenericFunction function;
@@ -90,37 +90,37 @@ typedef struct {
 	int meth;
 
 	/* Whether we need to copy to a buffer or not.*/
-	int needbuffer[MAX_ARGS];
+	int needbuffer[NPY_MAXARGS];
 	int leftover;
 	int ninnerloops;
 	int lastdim;
-	
+
 	/* Whether or not to swap */
-	int swap[MAX_ARGS];
+	int swap[NPY_MAXARGS];
 
 	/* Buffers for the loop */
-	char *buffer[MAX_ARGS];
+	char *buffer[NPY_MAXARGS];
 	int bufsize;
-	intp bufcnt;
-	char *dptr[MAX_ARGS];
+	npy_intp bufcnt;
+	char *dptr[NPY_MAXARGS];
 
 	/* For casting */
-	char *castbuf[MAX_ARGS];
-	PyArray_VectorUnaryFunc *cast[MAX_ARGS];
+	char *castbuf[NPY_MAXARGS];
+	PyArray_VectorUnaryFunc *cast[NPY_MAXARGS];
 
 	/* usually points to buffer but when a cast is to be
 	   done it switches for that argument to castbuf.
 	*/
-	char *bufptr[MAX_ARGS];  
+	char *bufptr[NPY_MAXARGS];
 
 	/* Steps filled in from iters or sizeof(item)
-	   depending on loop method. 
+	   depending on loop method.
 	*/
-	intp steps[MAX_ARGS];
+	npy_intp steps[NPY_MAXARGS];
 
         int obj;  /* This loop uses object arrays */
-        int notimplemented; /* The loop caused notimplemented */	
-        int objfunc; /* This loop calls object functions 
+        int notimplemented; /* The loop caused notimplemented */
+        int objfunc; /* This loop calls object functions
                         (an inner-loop function with argument types */
 } PyUFuncLoopObject;
 
@@ -133,8 +133,8 @@ typedef struct {
         PyArrayObject *ret;
 	PyArrayIterObject *rit;   /* Needed for Accumulate */
         int  outsize;
-	intp  index;
-	intp  size;
+	npy_intp  index;
+	npy_intp  size;
         char idptr[UFUNC_MAXIDENTITY];
 
 	/* The ufunc */
@@ -143,6 +143,7 @@ typedef struct {
 	/* The error handling */
 	int errormask;
 	PyObject *errobj;
+        int first;
 
         PyUFuncGenericFunction function;
         void *funcdata;
@@ -156,9 +157,9 @@ typedef struct {
         PyArray_VectorUnaryFunc *cast;
 
         char *bufptr[3];
-        intp steps[3];
+        npy_intp steps[3];
 
-        intp N;
+        npy_intp N;
         int  instrides;
         int  insize;
         char *inptr;
@@ -167,16 +168,17 @@ typedef struct {
         PyObject *decref;
 
         int obj;
+        int retbase;
 
 } PyUFuncReduceObject;
 
 
-#if defined(ALLOW_THREADS)
-#define LOOP_BEGIN_THREADS if (!(loop->obj)) {_save = PyEval_SaveThread();}
-#define LOOP_END_THREADS   if (!(loop->obj)) {PyEval_RestoreThread(_save);}
+#if NPY_ALLOW_THREADS
+#define NPY_LOOP_BEGIN_THREADS if (!(loop->obj)) {_save = PyEval_SaveThread();}
+#define NPY_LOOP_END_THREADS   if (!(loop->obj)) {PyEval_RestoreThread(_save);}
 #else
-#define LOOP_BEGIN_THREADS
-#define LOOP_END_THREADS
+#define NPY_LOOP_BEGIN_THREADS
+#define NPY_LOOP_END_THREADS
 #endif
 
 #define PyUFunc_One 1
@@ -195,16 +197,27 @@ typedef struct {
         PyObject *callable;
 } PyUFunc_PyFuncData;
 
+/* A linked-list of function information for
+   user-defined 1-d loops.
+ */
+typedef struct _loop1d_info {
+        PyUFuncGenericFunction func;
+        void *data;
+        int *arg_types;
+        struct _loop1d_info *next;
+} PyUFunc_Loop1d;
+
 
 #include "__ufunc_api.h"
 
 #define UFUNC_PYVALS_NAME "UFUNC_PYVALS"
 
-#define UFUNC_CHECK_ERROR(arg)                                         \
-	if (((arg)->obj && PyErr_Occurred()) ||                        \
-            ((arg)->errormask &&                                       \
-             PyUFunc_checkfperr((arg)->errormask,                      \
-                                (arg)->errobj)))                       \
+#define UFUNC_CHECK_ERROR(arg)                                          \
+	if (((arg)->obj && PyErr_Occurred()) ||                         \
+            ((arg)->errormask &&                                        \
+             PyUFunc_checkfperr((arg)->errormask,                       \
+                                (arg)->errobj,                          \
+                                &(arg)->first)))                        \
 		goto fail
 
 /* This code checks the IEEE status flags in a platform-dependent way */
@@ -226,9 +239,9 @@ typedef struct {
 		| ((IEEE_STATUS_UNF & fpstatus) ? UFUNC_FPE_UNDERFLOW : 0) \
 		| ((IEEE_STATUS_INV & fpstatus) ? UFUNC_FPE_INVALID : 0); \
 	}
-	
+
 /* MS Windows -----------------------------------------------------*/
-#elif defined(_MSC_VER) 
+#elif defined(_MSC_VER)
 
 #include <float.h>
 
@@ -245,11 +258,11 @@ typedef struct {
 #define isinf(x) ((_fpclass((double)(x)) == _FPCLASS_PINF) ||	\
 		  (_fpclass((double)(x)) == _FPCLASS_NINF))
 #define isfinite(x) (_finite((double) x))
-	
+
 /* Solaris --------------------------------------------------------*/
 /* --------ignoring SunOS ieee_flags approach, someone else can
 **         deal with that! */
-#elif defined(sun) || defined(__BSD__) || defined(__OpenBSD__)
+#elif defined(sun) || defined(__BSD__) || defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__)
 #include <ieeefp.h>
 
 #define UFUNC_CHECK_STATUS(ret) {				\
@@ -262,7 +275,7 @@ typedef struct {
 		| ((FP_X_INV & fpstatus) ? UFUNC_FPE_INVALID : 0);	\
 	(void) fpsetsticky(0);						\
 	}
-	
+
 #elif defined(linux) || defined(__APPLE__) || defined(__CYGWIN__) || defined(__MINGW32__)
 
 #if defined(__GLIBC__) || defined(__APPLE__) || defined(__MINGW32__)
@@ -284,7 +297,7 @@ typedef struct {
 
 #define generate_divbyzero_error() feraiseexcept(FE_DIVBYZERO)
 #define generate_overflow_error() feraiseexcept(FE_OVERFLOW)
-	
+
 #elif defined(_AIX)
 
 #include <float.h>
@@ -298,7 +311,7 @@ typedef struct {
 		| ((FP_OVERFLOW & fpstatus) ? UFUNC_FPE_OVERFLOW : 0)	\
 		| ((FP_UNDERFLOW & fpstatus) ? UFUNC_FPE_UNDERFLOW : 0) \
 		| ((FP_INVALID & fpstatus) ? UFUNC_FPE_INVALID : 0); \
-	fp_clr_flag( FP_DIV_BY_ZERO | FP_OVERFLOW | FP_UNDERFLOW | FP_INVALID); \
+	fp_swap_flag(0); \
 }
 
 #define generate_divbyzero_error() fp_raise_xcp(FP_DIV_BY_ZERO)
@@ -306,7 +319,7 @@ typedef struct {
 
 #else
 
-#define NO_FLOATING_POINT_SUPPORT 
+#define NO_FLOATING_POINT_SUPPORT
 #define UFUNC_CHECK_STATUS(ret) { \
     ret = 0;							     \
   }
@@ -327,7 +340,7 @@ static void generate_divbyzero_error(void) {
 	   return;
 	else /* should never be called */
 	   numeric_zero2 += 1;
-	return;	
+	return;
 }
 #endif
 
@@ -346,14 +359,6 @@ static void generate_overflow_error(void) {
 #endif
 
 
-#ifdef PY_ARRAY_TYPES_PREFIX
-#  undef CAT
-#  undef CAT2
-#  undef NS
-#  undef inpt
-#endif
-
-
 #ifdef __cplusplus
 }
 #endif
