@@ -4,17 +4,26 @@ Functions to operate on polynomials.
 
 __all__ = ['poly', 'roots', 'polyint', 'polyder', 'polyadd',
            'polysub', 'polymul', 'polydiv', 'polyval', 'poly1d',
-           'polyfit']
+           'polyfit', 'RankWarning']
 
 import re
+import warnings
 import numpy.core.numeric as NX
 
-from numpy.core import isscalar
+from numpy.core import isscalar, abs
+from numpy.lib.getlimits import finfo
 from numpy.lib.twodim_base import diag, vander
 from numpy.lib.shape_base import hstack, atleast_1d
 from numpy.lib.function_base import trim_zeros, sort_complex
 eigvals = None
 lstsq = None
+_single_eps = finfo(NX.single).eps
+_double_eps = finfo(NX.double).eps
+
+class RankWarning(UserWarning):
+    """Issued by polyfit when Vandermonde matrix is rank deficient.
+    """
+    pass
 
 def get_linalg_funcs():
     "Look for linear algebra functions in numpy"
@@ -30,13 +39,13 @@ def _eigvals(arg):
         get_linalg_funcs()
         return eigvals(arg)
 
-def _lstsq(X, y):
+def _lstsq(X, y, rcond):
     "Do least squares on the arguments"
     try:
-        return lstsq(X, y)
+        return lstsq(X, y, rcond)
     except TypeError:
         get_linalg_funcs()
-        return lstsq(X, y)
+        return lstsq(X, y, rcond)
 
 def poly(seq_of_zeros):
     """ Return a sequence representing a polynomial given a sequence of roots.
@@ -90,7 +99,11 @@ def roots(p):
         raise ValueError,"Input must be a rank-1 array."
 
     # find non-zero array entries
-    non_zero = NX.nonzero(NX.ravel(p))
+    non_zero = NX.nonzero(NX.ravel(p))[0]
+
+    # Return an empty array if polynomial is all zeros
+    if len(non_zero) == 0:
+        return NX.array([])
 
     # find the number of trailing zeros -- this is the number of roots at 0.
     trailing_zeros = len(p) - non_zero[-1] - 1
@@ -109,7 +122,7 @@ def roots(p):
         A[0, :] = -p[1:] / p[0]
         roots = _eigvals(A)
     else:
-        return NX.array([])
+        roots = NX.array([])
 
     # tack any zeros onto the back of the array
     roots = hstack((roots, NX.zeros(trailing_zeros, roots.dtype)))
@@ -165,11 +178,31 @@ def polyder(p, m=1):
             val = poly1d(val)
         return val
 
-def polyfit(x, y, N):
-    """
+def polyfit(x, y, deg, rcond=None, full=False):
+    """Least squares polynomial fit.
 
-    Do a best fit polynomial of order N of y to x.  Return value is a
-    vector of polynomial coefficients [pk ... p1 p0].  Eg, for N=2
+    Required arguments
+
+        x -- vector of sample points
+        y -- vector or 2D array of values to fit
+        deg -- degree of the fitting polynomial
+
+    Keyword arguments
+
+        rcond -- relative condition number of the fit (default len(x)*eps)
+        full -- return full diagnostic output (default False)
+
+    Returns
+
+        full == False -- coefficients
+        full == True -- coefficients, residuals, rank, singular values, rcond.
+
+    Warns
+
+        RankWarning -- if rank is reduced and not full output
+
+    Do a best fit polynomial of degree 'deg' of 'x' to 'y'.  Return value is a
+    vector of polynomial coefficients [pk ... p1 p0].  Eg, for n=2
 
       p2*x0^2 +  p1*x0 + p0 = y1
       p2*x1^2 +  p1*x1 + p0 = y1
@@ -191,7 +224,41 @@ def polyfit(x, y, N):
 
       p = (XT*X)^-1 * XT * y
 
-    where XT is the transpose of X and -1 denotes the inverse.
+    where XT is the transpose of X and -1 denotes the inverse. However, this
+    method is susceptible to rounding errors and generally the singular value
+    decomposition is preferred and that is the method used here. The singular
+    value method takes a paramenter, 'rcond', which sets a limit on the
+    relative size of the smallest singular value to be used in solving the
+    equation. This may result in lowering the rank of the Vandermonde matrix,
+    in which case a RankWarning is issued. If polyfit issues a RankWarning, try
+    a fit of lower degree or replace x by x - x.mean(), both of which will
+    generally improve the condition number. The routine already normalizes the
+    vector x by its maximum absolute value to help in this regard. The rcond
+    parameter may also be set to a value smaller than its default, but this may
+    result in bad fits. The current default value of rcond is len(x)*eps, where
+    eps is the relative precision of the floating type being used, generally
+    around 1e-7 and 2e-16 for IEEE single and double precision respectively.
+    This value of rcond is fairly conservative but works pretty well when x -
+    x.mean() is used in place of x.
+
+    The warnings can be turned off by:
+
+    >>> import numpy
+    >>> import warnings
+    >>> warnings.simplefilter('ignore',numpy.RankWarning)
+
+    DISCLAIMER: Power series fits are full of pitfalls for the unwary once the
+    degree of the fit becomes large or the interval of sample points is badly
+    centered. The basic problem is that the powers x**n are generally a poor
+    basis for the functions on the sample interval with the result that the
+    Vandermonde matrix is ill conditioned and computation of the polynomial
+    values is sensitive to coefficient error. The quality of the resulting fit
+    should be checked against the data whenever the condition number is large,
+    as the quality of polynomial fits *can not* be taken for granted. If all
+    you want to do is draw a smooth curve through the y values and polyfit is
+    not doing the job, try centering the sample range or look into
+    scipy.interpolate, which includes some nice spline fitting functions that
+    may be of use.
 
     For more info, see
     http://mathworld.wolfram.com/LeastSquaresFittingPolynomial.html,
@@ -201,13 +268,50 @@ def polyfit(x, y, N):
     See also polyval
 
     """
-    x = NX.asarray(x)+0.
-    y = NX.asarray(y)+0.
-    y = NX.reshape(y, (len(y), 1))
-    X = vander(x, N+1)
-    c, resids, rank, s = _lstsq(X, y)
-    c.shape = (N+1,)
-    return c
+    order = int(deg) + 1
+    x = NX.asarray(x) + 0.0
+    y = NX.asarray(y) + 0.0
+
+    # check arguments.
+    if deg < 0 :
+        raise ValueError, "expected deg >= 0"
+    if x.ndim != 1 or x.size == 0:
+        raise TypeError, "expected non-empty vector for x"
+    if y.ndim < 1 or y.ndim > 2 :
+        raise TypeError, "expected 1D or 2D array for y"
+    if x.shape[0] != y.shape[0] :
+        raise TypeError, "expected x and y to have same length"
+
+    # set rcond
+    if rcond is None :
+        xtype = x.dtype
+        if xtype == NX.single or xtype == NX.csingle :
+            rcond = len(x)*_single_eps
+        else :
+            rcond = len(x)*_double_eps
+
+    # scale x to improve condition number
+    scale = abs(x).max()
+    if scale != 0 :
+        x /= scale
+
+    # solve least squares equation for powers of x
+    v = vander(x, order)
+    c, resids, rank, s = _lstsq(v, y, rcond)
+
+    # warn on rank reduction, which indicates an ill conditioned matrix
+    if rank != order and not full:
+        msg = "Polyfit may be poorly conditioned"
+        warnings.warn(msg, RankWarning)
+
+    # scale returned coefficients
+    if scale != 0 :
+        c /= vander([scale], order)[0]
+
+    if full :
+        return c, resids, rank, s, rcond
+    else :
+        return c
 
 
 
@@ -244,7 +348,7 @@ def polyadd(a1, a2):
     a2 = atleast_1d(a2)
     diff = len(a2) - len(a1)
     if diff == 0:
-        return a1 + a2
+        val = a1 + a2
     elif diff > 0:
         zr = NX.zeros(diff, a1.dtype)
         val = NX.concatenate((zr, a1)) + a2
@@ -263,7 +367,7 @@ def polysub(a1, a2):
     a2 = atleast_1d(a2)
     diff = len(a2) - len(a1)
     if diff == 0:
-        return a1 - a2
+        val = a1 - a2
     elif diff > 0:
         zr = NX.zeros(diff, a1.dtype)
         val = NX.concatenate((zr, a1)) - a2
@@ -542,3 +646,8 @@ class poly1d(object):
         """Return the mth derivative of this polynomial.
         """
         return poly1d(polyder(self.coeffs, m=m))
+
+# Stuff to do on module import
+
+warnings.simplefilter('always',RankWarning)
+
